"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const schema_1 = require("./src/schema");
class MongoLocksError extends Error {
    constructor(message) {
        super(message);
    }
}
exports.MongoLocksError = MongoLocksError;
const makeLockId = array => Array.prototype.slice.apply(array).join("-");
class LockManager {
    constructor() {
        this.Locks = null;
        this.MongoLocksError = MongoLocksError;
        this.noop = () => { };
    }
    init(connection, options) {
        options = options || {};
        this.Locks = connection.model(options.collection || 'Locks', schema_1.default);
    }
    lock(...actions) {
        if (!this.Locks) {
            throw new MongoLocksError("You must initialize mongo-locks with a mongoose connection");
        }
        const lockId = makeLockId(actions);
        const l = new this.Locks();
        l.action = lockId;
        return l.save().then(() => () => this.Locks.remove({ action: lockId }).exec());
    }
    free(...actions) {
        if (!this.Locks) {
            throw new MongoLocksError("You must initialize mongo-locks with a mongoose connection");
        }
        return this.Locks.remove(makeLockId(actions)).exec();
    }
}
exports.default = new LockManager();
